package com.ruoyi.inventory.service.impl;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.logging.SimpleFormatter;
import java.util.stream.Collectors;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.json.JSONUtil;
import com.ruoyi.asset.domain.TbAssetInformation;
import com.ruoyi.asset.mapper.TbAssetInformationMapper;
import com.ruoyi.asset.mapper.TbLocationMapper;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.inventory.domain.TagInfo;
import com.ruoyi.inventory.domain.TbInventoryDetail;
import com.ruoyi.inventory.domain.dto.TakeStockDTO;
import com.ruoyi.inventory.domain.dto.TbAssetInventoryDTO;
import com.ruoyi.inventory.mapper.TbInventoryDetailMapper;
import com.ruoyi.utils.IDUtil;
import com.ruoyi.utils.RecursionUtil;
import org.apache.commons.collections.map.HashedMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.inventory.mapper.TbAssetInventoryMapper;
import com.ruoyi.inventory.domain.TbAssetInventory;
import com.ruoyi.inventory.service.ITbAssetInventoryService;

/**
 * 资产盘点记录Service业务层处理
 * 
 * @author ydl
 * @date 2023-06-01
 */
@Service
public class TbAssetInventoryServiceImpl implements ITbAssetInventoryService
{
    @Autowired
    private TbAssetInventoryMapper tbAssetInventoryMapper;

    @Autowired
    private TbAssetInformationMapper informationMapper;

    @Autowired
    private TbInventoryDetailMapper tbInventoryDetailMapper;

    @Autowired
    private TbLocationMapper locationMapper;

    /**
     * 查询资产盘点记录
     * 
     * @param id 资产盘点记录主键
     * @return 资产盘点记录
     */
    @Override
    public TbAssetInventory selectTbAssetInventoryById(Long id)
    {
        return tbAssetInventoryMapper.selectTbAssetInventoryById(id);
    }

    @Override
    public TbAssetInventory selectTbAssetInventoryByNum(String num) {
        return tbAssetInventoryMapper.selectTbAssetInventoryByNumber(num);
    }

    /**
     * 查询资产盘点记录列表
     * 
     * @param tbAssetInventory 资产盘点记录
     * @return 资产盘点记录
     */
    @Override
    public List<TbAssetInventory> selectTbAssetInventoryList(TbAssetInventory tbAssetInventory)
    {
        return tbAssetInventoryMapper.selectTbAssetInventoryList(tbAssetInventory);
    }

    /**
     * 新增资产盘点记录
     * 
     * @param tbAssetInventory 资产盘点记录
     * @return 结果
     */
    @Override
    public int insertTbAssetInventory(TbAssetInventory tbAssetInventory)
    {
        tbAssetInventory.setOrderNumber(IDUtil.getUUID1());
        return tbAssetInventoryMapper.insertTbAssetInventory(tbAssetInventory);
    }

    //手持机盘点
    @Override
    public int addByTakeStock(List<TakeStockDTO> _dto){
        int result = 0;
        for (TakeStockDTO dto : _dto) {
            //解析实体
            List<TagInfo> list = dto.getAssetList();
            String location = dto.getInventoryLocation();
            String orderNumber = dto.getOrderNumber();
            //提取epc rfid
            ArrayList<String> epcList = new ArrayList<>();
            for (TagInfo tagInfo : list) {
                String epc = tagInfo.getEpc();
                // 删除字符串中前导的零
                epc = epc.replaceFirst("^0+", "");
                epcList.add(epc);
            }
            //根据位置获取包含自身及子位置,递归
            List<String> locations = new RecursionUtil().getNumberWithChildren(locationMapper, location);
            //获取所在位置内的资产
            List<String> rfidList = informationMapper.selectNumberByLocations(locations);
            //对比盘点
            List<String> intersection = new ArrayList<>(rfidList);
            intersection.retainAll(epcList);
            if (intersection.size() < rfidList.size()) {
                //盘亏
                if (!orderNumber.isEmpty()) {
                    result = takeStock(orderNumber, epcList, 2L);
                } else {
                    result = takeStock2(location, 2L, epcList);
                }
            } else if (list.size() > intersection.size()) {
                //盘盈
                if (!orderNumber.isEmpty()) {
                    result = takeStock(orderNumber, epcList, 1L);
                } else {
                    result = takeStock2(location, 1L, epcList);
                }
            } else if (intersection.size() == rfidList.size()) {
                //正常
                if (!orderNumber.isEmpty()) {
                    result = takeStock(orderNumber, epcList, 3L);
                } else {
                    result = takeStock2(location, 3L, epcList);
                }
            } else {
                result = 0;
            }
        }
        return result;
    }

    /**
     * 修改资产盘点记录
     * 
     * @param tbAssetInventory 资产盘点记录
     * @return 结果
     */
    @Override
    public int updateTbAssetInventory(TbAssetInventory tbAssetInventory)
    {
        return tbAssetInventoryMapper.updateTbAssetInventory(tbAssetInventory);
    }

    /**
     * 批量删除资产盘点记录
     * 
     * @param ids 需要删除的资产盘点记录主键
     * @return 结果
     */
    @Override
    public int deleteTbAssetInventoryByIds(Long[] ids)
    {
        return tbAssetInventoryMapper.deleteTbAssetInventoryByIds(ids);
    }

    /**
     * 删除资产盘点记录信息
     * 
     * @param id 资产盘点记录主键
     * @return 结果
     */
    @Override
    public int deleteTbAssetInventoryById(Long id)
    {
        return tbAssetInventoryMapper.deleteTbAssetInventoryById(id);
    }

    @Override
    public List<TbAssetInventoryDTO> selectTbAssetInventoryDTOList(TbAssetInventoryDTO dto) {
        return tbAssetInventoryMapper.selectTbAssetInventoryDTOList(dto);
    }

    private int takeStock(String orderNumber, List<String> list, Long result){
        TbAssetInventory inventory = tbAssetInventoryMapper.selectTbAssetInventoryByNumber(orderNumber);
        if (inventory.getRecordStatus() == 1L) {
            //插入盘点
            inventory.setInventoryStatus(2L);
            inventory.setInventoryResult(result);
            //插入明细
            setDetail(orderNumber, list);
            return updateTbAssetInventory(inventory);
        }else {
            return 0;
        }
    }

    private int takeStock2(String location, Long result, List<String> list){
        TbAssetInventory inventory = new TbAssetInventory();
        inventory.setInventoryStatus(2L);
        inventory.setInventoryResult(result);
        inventory.setRecordStatus(1L);
        inventory.setInventoryLocation(location);
        inventory.setInventoryDate(new Date());
        SimpleDateFormat formatter = new SimpleDateFormat("yyMMdd HH:mm:ss");
        String format = formatter.format(new Date());
        inventory.setName("线下自测"+format);
        int i = insertTbAssetInventory(inventory);
        if(i < 1){
            throw new ServiceException("生成盘点记录失败");
        }
        String orderNumber = inventory.getOrderNumber();
        //插入盘点明细
        setDetail(orderNumber, list);
        return 1;
    }

    private void setDetail(String orderNumber, List<String> list){
        //该位置下的资产
        TbAssetInventory _inventory = tbAssetInventoryMapper.selectTbAssetInventoryByNumber(orderNumber);
        TbAssetInformation information = new TbAssetInformation();
        String location = _inventory.getInventoryLocation();
        information.setLocationNumber(location);
        List<String> childrens = new RecursionUtil().getNumberWithChildren(locationMapper, location);
        information.setNlist(childrens);
        List<TbAssetInformation> informations = informationMapper.selectTbAssetInformationList(information);
        if (informations == null || informations.isEmpty() ){
            return ;
        }
        //该位置下资产rfid list
        List<String> _list = informations.stream().map(TbAssetInformation::getNumber).collect(Collectors.toList());

        //已盘点,1
        List<String> green = (List) CollectionUtil.intersection(_list, list);
        //未盘点,2
        List<String> red = new ArrayList<>(_list);
        red.removeAll(list);
        //异常(多余),0
        List<String> yellow = new ArrayList<>(list);
        yellow.removeAll(_list);
        //插入详细盘点信息
        TbInventoryDetail detail = new TbInventoryDetail();
        //已盘
        setInventoryDetail(green, detail, orderNumber, 1L);
        //未盘
        setInventoryDetail(red, detail, orderNumber, 2L);
        //异常
        setInventoryDetail(yellow, detail, orderNumber, 0L);
    }

    private void setInventoryDetail(List<String> list, TbInventoryDetail detail, String orderNumber, Long isUpdate){
        for (String number : list) {
            detail.setOrderNumber(orderNumber);
            detail.setInventoryMetadata(JSONUtil.toJsonStr(informationMapper.selectTbAssetInformationByAssetNumber(number)));
            detail.setIsUpdateData(isUpdate);
            tbInventoryDetailMapper.insertTbInventoryDetail(detail);
        }
    }

}
